==> IceWarpServer-9.3.2_RHEL4.tar.gz: html/webmail/server/inc/imap/item.php <==
<?php

require_once('inc/mailparse.php');
require_once('inc/cache/item.php');
require_once('inc/imap/imap.php');

/**
  @class IMAPItem
  @brief Class representing an IMAP item.
  @author Martin Zlomek
  @date 09-02-2006
 */
class IMAPItem extends CacheItem {
	/**
	  @brief Create a new IMAP item object.
	  @param $folder Parent folder object of the item.
	  @param $itemID ID of the item.
	  @param $rid Remote ID of the item.
	  @param $size Size of the item.
	  @param $date Date of the item.
	  @param $flags Flags of the item.
	  @param $from From header of the item.
	  @param $to To header of the item.
	  @param $subject Subject of the item.
	  @param $staticFlags Static flags of item (cannot be set by webmail client).
	  @param $priority Priority of the item.
	  @param $color Color of the item.
	  @param $sMimeStatus S-Mime status of the item (item is signed/verified).
	  @param $hasAttachments Item has attachments.
	 */
	public function __construct($folder, $itemID, $rid,$size, $date, $flags, 
								$from, $to, $subject, $staticFlags, $priority, $color,
								$sMimeStatus, $hasAttachments)
	{
		
  		parent::__construct($folder, $itemID, $rid, $size, $date, $flags, 
							$from, $to, $subject, $staticFlags, $priority, $color,
							$sMimeStatus, $hasAttachments);	
      $this->wmclass = 'M';
  }

	/**
	  @brief Decode an item from IMAP format.
	  @param[in,out] $item The IMAP item.
	  @param $staticFlags Value of the static flags
	  */
	public static function decode(&$item, $staticFlags = array())
	{
	  $item->from = MailParse::decodeMimeHeader($item->from);
		$item->to = MailParse::decodeMimeHeader($item->to);
		$item->subject = MailParse::decodeMimeHeader($item->subject);

    $item->flags   = self::decodeFlags($item);
    
    if($item->flags & ITEM::FLAG_FLAGGED){
      $item->color = 1;
    }else{
      $item->color = 'Z';
    }

		$item->priority  = isset($item->priority) ? intval($item->priority) : Item::NORMAL_PRIORITY;
		$item->staticFlags = isset($staticFlags['flags']) ? $staticFlags['flags'] : 0;
		$item->sMimeStatus = isset($staticFlags['smime_status']) ? $staticFlags['smime_status'] : 0;
		$item->hasAttachments = isset($staticFlags['has_attachment']) ? $staticFlags['has_attachment'] : false;
		//$item->date = @strftime("%Y-%m-%d %H:%M:%S",$item->internaldate);
		//if(!$item->date) $item->date = strftime("%Y-%m-%d %H:%M:%S",UNIX1900);
		$item->date = $item->internaldate;
		if(!$item->date) $item->date = time();

  } 

	/**
	  @brief Create a new item.
	  @param $folder Parent folder object of the item.
	  @param $message The message of the item.
	 */
	// todo: synchronise, not compatible
	public static function create($folder, $message,$file = false,$bNoCache = false)
	{
		$account = $folder->account;
		/* Create the item on the IMAP server. */
		$imap = IMAP::instance($account);
    
    $newID = $bNoCache;
    
		$item = $imap->createMessage($folder->name, $message, $file, $newID);

		$staticFlags = parent::getStaticFlags($message,true,$file);
		self::decode($item, $staticFlags);

    if($bNoCache)
      $result = $newID;
    else
		/* Create the item in the item cache. */
		$result = parent::create($folder, $item, $message,array(),$file);
	
    return $result;
  }
	
	/**
	  @brief Delete the item.
	 */
	public function delete($cache = array())
	{
		$folder  = &$this->folder;
		$account = &$folder->account;

		/* Delete the item from the IMAP server. */
    if (!$cache['imap'])
      $cache['imap'] = IMAP::instance($account);
		$cache['imap']->deleteMessage($folder->name, $this->rid);

		/* Delete the item from the item cache. */
		parent::delete($cache);
	}
	
	
	/**
	  @brief Edit the message of the item.
	  @param $message New message of the item.
	 */
	// todo: synchronise, not compatible
	public function edit($message,$file = false)
	{
		$folder = $this->folder;
		$account = $folder->account;

		/* Delete the original message on IMAP server and create a new one. */
		$imap = IMAP::instance($account);

    $imap->deleteMessage($folder->name, $this->rid);
    
    $item = $imap->createMessage($folder->name, $message, $file);
    $imap->setFlags($folder->name,$item->uid,"\\Seen");

    $staticFlags = parent::getStaticFlags($message,true,$file);
		self::decode($item, $staticFlags);

		/* Edit the item in the item cache. */
	  return parent::edit($item, $message, array(), $file);
	}

	/**
	  @brief Move an item to another folder.
	  @param $folder Destination folder object.
	 */
	public function move($folder, $cache = array())
	{
		$account = $folder->account;

		/* Move the item to another folder on the IMAP server. */
		if (!$cache['imap'])
    $cache['imap'] = IMAP::instance($account);
		$this->rid = $cache['imap']->moveMessage($this->folder->name, $this->rid,
										$folder->name);

		/* Move the item to another folder in the item cache. */
		parent::move($folder, $cache);
	}
	
	
	/**
	  @brief Copy an item to another folder.
	  @param $folder Destination folder object.
	 */
	// todo: synchronise, not compatible
	public function copy($folder, $cache = array())
	{
		$account = $folder->account;

		
		if(!$cache['cache']){
       $cache['cache'] = new Cache($account->user);
       $fCache = false;
    } else $fCache = true;
    
    /* Get original message */  
    $message = $cache['cache']->getMessage($account->accountID, $this->folder->folderID, $this->itemID);

		/* Move the item to another folder on the IMAP server. */
		if (!$cache['imap'])
    $cache['imap'] = IMAP::instance($account);
		$this->rid = $cache['imap']->moveMessage($this->folder->name, $this->rid,
										$folder->name, true);

		/* Move the item to another folder in the item cache. */
		parent::copy($folder, $message, $cache);
	}

	/**
	  @brief Set specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	// todo: synchronise, not compatible
	public function setFlags($flags, $field = 'flags')
	{
		// if all required flags are set already, do nothing
		if (!$this->checkSetFlags($flags, $field))
			return;
		
		if ($field == 'flags') {
			$folder  = $this->folder;
			$account = $folder->account;
	
			/* Set flags on the IMAP item. */
			$imap = IMAP::instance($account);
			$imap->setFlags($folder->name, $this->rid, self::encodeFlags($flags));
		}

		/* Set flags on the cached item. */
		parent::setFlags($flags, $field);
	}
	
	public function markAsRead()
	{
		$this->setFlags(Item::FLAG_SEEN);
	}
	
  public function markAsUnread()
	{
		$this->unsetFlags(Item::FLAG_SEEN);
	}
	/**
	  @brief Unset specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	// todo: synchronise, not compatible
	protected function unsetFlags($flags, $field = 'flags')
	{
		// if all required flags are unset already, do nothing
		if (!$this->checkUnsetFlags($flags, $field))
			return;
		
		if ($field == 'flags') {
			$folder  = $this->folder;
			$account = $folder->account;
	
			/* Unset flags on the IMAP item. */
			$imap = IMAP::instance($account);
			$imap->unsetFlags($folder->name, $this->rid, self::encodeFlags($flags));
		}

		/* Unset flags on the cached item. */
		parent::unsetFlags($flags, $field);
	}
	
	/**
	  @brief Convert flags to IMAP format.
	  @param $flags The flags.
	  @return The converted flags.
	 */
	public static function encodeFlags($flags)
	{
		$imapFlags = '';
		if ($flags & Item::FLAG_ANSWERED)
			$imapFlags .= '\\Answered';
		if ($flags & Item::FLAG_DELETED)
			$imapFlags .= '\\Deleted';
		if ($flags & Item::FLAG_DRAFT)
			$imapFlags .= '\\Draft';
		if ($flags & Item::FLAG_FLAGGED)
			$imapFlags .= '\\Flagged';
		if ($flags & Item::FLAG_SEEN)
			$imapFlags .= '\\Seen';
		if ($flags & Item::FLAG_FORWARDED)
			$imapFlags .= '$Forwarded';

		return $imapFlags;
	}

	/**
	  @brief Convert flags from IMAP format.
	  @param $item The IMAP item.
	  @return The converted flags.
	 */
	public static function decodeFlags($item)
	{
		$flags = 0;
		$flags |= $item->answered ? Item::FLAG_ANSWERED : 0;
		$flags |= $item->deleted ? Item::FLAG_DELETED : 0;
		$flags |= $item->draft ? Item::FLAG_DRAFT : 0;
		$flags |= $item->flagged ? Item::FLAG_FLAGGED : 0;
		$flags |= $item->recent ? Item::FLAG_RECENT : 0;
		$flags |= $item->seen ? Item::FLAG_SEEN : 0;
    $flags |= $item->forwarded ? Item::FLAG_FORWARDED : 0;
    
		return $flags;
	}

	/**
	  @brief Sync the flags of the item.
	 */
	private function sync()
	{
		$folder  = $this->folder;
		$account = $folder->account;

		$imap = IMAP::instance($account);
						 
		$item = $imap->getItemDetails($folder->name, $this->rid, false);
		$flags = self::decodeFlags($item);

//		$this->setFullFlags($flags);
	}
	
	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	public function getMessage()
	{
		$folder  = $this->folder;
		$account = $folder->account;

		/* Get cached message. */
		$message = parent::getMessage();

		if ($message === false) {
			/* Get IMAP message. */
			$imap = IMAP::instance($account);
			$message = $imap->getMessage($folder->name, $this->rid);

			/* Cache the message. */
			parent::createMessage($message);			
			
			$this->sync();
		} else {
			// mark message as read (unleass it is read already)
			$this->setFlags(Item::FLAG_SEEN);
		}
		return $message;
	}

	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	public function autoCreateMessage($filename)
	{
		$folder  = $this->folder;
		$account = $folder->account;

    /* Check cached message. */
		if (parent::getMessage() === false) {

			/* Get IMAP message. */
			$imap = IMAP::instance($account);
			
			$dir = dirname($filename);
			
			if(!is_dir($dir))
			 System::mkdir_r($dir);

      //Cache the message
			$imap->saveMessageToFile($folder->name, $this->rid, $filename);
			Tools::checkMessageEnd($filename);
			$this->sync();
		} else {
			// mark message as read (unleass it is read already)
			$this->setFlags(Item::FLAG_SEEN);
		}
		return true;
	}
  	
	public function getBody()
	{
		  
	}
}

?>

==> IceWarpServer-9.4.0_RHEL4.tar.gz: html/webmail/server/inc/imap/item.php <==
<?php

require_once('inc/mailparse.php');
require_once('inc/cache/item.php');
require_once('inc/imap/imap.php');

/**
  @class IMAPItem
  @brief Class representing an IMAP item.
  @author Martin Zlomek
  @date 09-02-2006
 */
class IMAPItem extends CacheItem {
	/**
	  @brief Create a new IMAP item object.
	  @param $folder Parent folder object of the item.
	  @param $itemID ID of the item.
	  @param $rid Remote ID of the item.
	  @param $size Size of the item.
	  @param $date Date of the item.
	  @param $flags Flags of the item.
	  @param $from From header of the item.
	  @param $to To header of the item.
	  @param $subject Subject of the item.
	  @param $staticFlags Static flags of item (cannot be set by webmail client).
	  @param $priority Priority of the item.
	  @param $color Color of the item.
	  @param $sMimeStatus S-Mime status of the item (item is signed/verified).
	  @param $hasAttachments Item has attachments.
	 */
	public function __construct($folder, $itemID, $rid,$size, $date, $flags, 
								$from, $to, $subject, $staticFlags, $priority, $color,
								$sMimeStatus, $hasAttachments)
	{
		
  		parent::__construct($folder, $itemID, $rid, $size, $date, $flags, 
							$from, $to, $subject, $staticFlags, $priority, $color,
							$sMimeStatus, $hasAttachments);	
      $this->wmclass = 'M';
  }

	/**
	  @brief Decode an item from IMAP format.
	  @param[in,out] $item The IMAP item.
	  @param $staticFlags Value of the static flags
	  */
	public static function decode(&$item, $staticFlags = array())
	{
		$item->from = Tools::fixHeader( MailParse::decodeMimeHeader( $item->from ) );
		$item->to = Tools::fixHeader( MailParse::decodeMimeHeader( $item->to ) );
		$item->subject = trim( MailParse::decodeMimeHeader($item->subject) );
		$item->flags   = self::decodeFlags($item);
	    if($item->flags & ITEM::FLAG_FLAGGED){
	      $item->color = 1;
	    }else{
	      $item->color = 'Z';
	    }
		$item->priority  = isset($item->priority) ? intval($item->priority) : Item::NORMAL_PRIORITY;
		$item->staticFlags = isset($staticFlags['flags']) ? $staticFlags['flags'] : 0;
		$item->sMimeStatus = isset($staticFlags['smime_status']) ? $staticFlags['smime_status'] : 0;
		$item->hasAttachments = isset($staticFlags['has_attachment']) ? $staticFlags['has_attachment'] : false;
		//$item->date = @strftime("%Y-%m-%d %H:%M:%S",$item->internaldate);
		//if(!$item->date) $item->date = strftime("%Y-%m-%d %H:%M:%S",UNIX1900);
		$item->date = $item->internaldate;
		if(!$item->date) $item->date = time();
  } 

	/**
	  @brief Create a new item.
	  @param $folder Parent folder object of the item.
	  @param $message The message of the item.
	 */
	// todo: synchronise, not compatible
	public static function create($folder, $message,$file = false,$bNoCache = false)
	{
		$account = $folder->account;
		/* Create the item on the IMAP server. */
		$imap = IMAP::instance($account);
    
    $newID = $bNoCache;
    
		$item = $imap->createMessage($folder->name, $message, $file, $newID);

		$staticFlags = parent::getStaticFlags($message,true,$file);
		self::decode($item, $staticFlags);

    if($bNoCache)
      $result = $newID;
    else
		/* Create the item in the item cache. */
		$result = parent::create($folder, $item, $message,array(),$file);
	
    return $result;
  }
	
	/**
	  @brief Delete the item.
	 */
	public function delete($cache = array())
	{
		$folder  = &$this->folder;
		$account = &$folder->account;

		/* Delete the item from the IMAP server. */
    if (!$cache['imap'])
      $cache['imap'] = IMAP::instance($account);
		$cache['imap']->deleteMessage($folder->name, $this->rid);

		/* Delete the item from the item cache. */
		parent::delete($cache);
	}
	
	
	/**
	  @brief Edit the message of the item.
	  @param $message New message of the item.
	 */
	// todo: synchronise, not compatible
	public function edit($message,$file = false)
	{
		$folder = $this->folder;
		$account = $folder->account;

		/* Delete the original message on IMAP server and create a new one. */
		$imap = IMAP::instance($account);

    $imap->deleteMessage($folder->name, $this->rid);
    
    $item = $imap->createMessage($folder->name, $message, $file);
    $imap->setFlags($folder->name,$item->uid,"\\Seen");

    $staticFlags = parent::getStaticFlags($message,true,$file);
		self::decode($item, $staticFlags);

		/* Edit the item in the item cache. */
	  return parent::edit($item, $message, array(), $file);
	}

	/**
	  @brief Move an item to another folder.
	  @param $folder Destination folder object.
	 */
	public function move($folder, $cache = array())
	{
		$account = $folder->account;

		/* Move the item to another folder on the IMAP server. */
		if (!$cache['imap'])
    $cache['imap'] = IMAP::instance($account);
		$this->rid = $cache['imap']->moveMessage($this->folder->name, $this->rid,
										$folder->name);

		/* Move the item to another folder in the item cache. */
		parent::move($folder, $cache);
	}
	
	
	/**
	  @brief Copy an item to another folder.
	  @param $folder Destination folder object.
	 */
	// todo: synchronise, not compatible
	public function copy($folder, $cache = array())
	{
		$account = $folder->account;

		
		if(!$cache['cache']){
       $cache['cache'] = Cache::instance($account->user);
       $fCache = false;
    } else $fCache = true;
    
    /* Get original message */  
    $message = $cache['cache']->getMessage($account->accountID, $this->folder->folderID, $this->itemID);

		/* Move the item to another folder on the IMAP server. */
		if (!$cache['imap'])
    $cache['imap'] = IMAP::instance($account);
		$this->rid = $cache['imap']->moveMessage($this->folder->name, $this->rid,
										$folder->name, true);

		/* Move the item to another folder in the item cache. */
		parent::copy($folder, $message, $cache);
	}

	/**
	  @brief Set specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	// todo: synchronise, not compatible
	public function setFlags($flags, $field = 'flags')
	{
		// if all required flags are set already, do nothing
		if (!$this->checkSetFlags($flags, $field))
			return;
		
		if ($field == 'flags') {
			$folder  = $this->folder;
			$account = $folder->account;
	
			/* Set flags on the IMAP item. */
			$imap = IMAP::instance($account);
			$imap->setFlags($folder->name, $this->rid, self::encodeFlags($flags));
		}

		/* Set flags on the cached item. */
		parent::setFlags($flags, $field);
	}
	
	public function markAsRead()
	{
		$this->setFlags(Item::FLAG_SEEN);
	}
	
  public function markAsUnread()
	{
		$this->unsetFlags(Item::FLAG_SEEN);
	}
	/**
	  @brief Unset specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	// todo: synchronise, not compatible
	protected function unsetFlags($flags, $field = 'flags')
	{
		// if all required flags are unset already, do nothing
		if (!$this->checkUnsetFlags($flags, $field))
			return;
		
		if ($field == 'flags') {
			$folder  = $this->folder;
			$account = $folder->account;
	
			/* Unset flags on the IMAP item. */
			$imap = IMAP::instance($account);
			$imap->unsetFlags($folder->name, $this->rid, self::encodeFlags($flags));
		}

		/* Unset flags on the cached item. */
		parent::unsetFlags($flags, $field);
	}
	
	/**
	  @brief Convert flags to IMAP format.
	  @param $flags The flags.
	  @return The converted flags.
	 */
	public static function encodeFlags($flags)
	{
		$imapFlags = '';
		if ($flags & Item::FLAG_ANSWERED)
			$imapFlags .= '\\Answered';
		if ($flags & Item::FLAG_DELETED)
			$imapFlags .= '\\Deleted';
		if ($flags & Item::FLAG_DRAFT)
			$imapFlags .= '\\Draft';
		if ($flags & Item::FLAG_FLAGGED)
			$imapFlags .= '\\Flagged';
		if ($flags & Item::FLAG_SEEN)
			$imapFlags .= '\\Seen';
		if ($flags & Item::FLAG_FORWARDED)
			$imapFlags .= '$Forwarded';

		return $imapFlags;
	}

	/**
	  @brief Convert flags from IMAP format.
	  @param $item The IMAP item.
	  @return The converted flags.
	 */
	public static function decodeFlags($item)
	{
		$flags = 0;
		$flags |= $item->answered ? Item::FLAG_ANSWERED : 0;
		$flags |= $item->deleted ? Item::FLAG_DELETED : 0;
		$flags |= $item->draft ? Item::FLAG_DRAFT : 0;
		$flags |= $item->flagged ? Item::FLAG_FLAGGED : 0;
		$flags |= $item->recent ? Item::FLAG_RECENT : 0;
		$flags |= $item->seen ? Item::FLAG_SEEN : 0;
    $flags |= $item->forwarded ? Item::FLAG_FORWARDED : 0;
    
		return $flags;
	}

	/**
	  @brief Sync the flags of the item.
	 */
	private function sync()
	{
		$folder  = $this->folder;
		$account = $folder->account;

		$imap = IMAP::instance($account);
						 
		$item = $imap->getItemDetails($folder->name, $this->rid, false);
		$flags = self::decodeFlags($item);

//		$this->setFullFlags($flags);
	}
	
	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	public function getMessage()
	{
		$folder  = $this->folder;
		$account = $folder->account;

		/* Get cached message. */
		$message = parent::getMessage();

		if ($message === false) {
			/* Get IMAP message. */
			$imap = IMAP::instance($account);
			$message = $imap->getMessage($folder->name, $this->rid);

			/* Cache the message. */
			parent::createMessage($message);			
			
			$this->sync();
		} else {
			// mark message as read (unleass it is read already)
			$this->setFlags(Item::FLAG_SEEN);
		}
		return $message;
	}

	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	public function autoCreateMessage($filename)
	{
		$folder  = $this->folder;
		$account = $folder->account;

    /* Check cached message. */
		if (parent::getMessage() === false) {

			/* Get IMAP message. */
			$imap = IMAP::instance($account);
			
			$dir = dirname($filename);
			
			if(!is_dir($dir))
			 System::mkdir_r($dir);

      //Cache the message
			$imap->saveMessageToFile($folder->name, $this->rid, $filename);
			Tools::checkMessageEnd($filename);
			$this->sync();
		} else {
			// mark message as read (unleass it is read already)
			$this->setFlags(Item::FLAG_SEEN);
		}
		return true;
	}
  	
	public function getBody()
	{
		  
	}
}

?>
